﻿using NServiceBus;
using System;
using System.Configuration;
using System.Linq;

namespace DCKiss.IMG.NSBSagaUpdate.API.EnqueueMessage
{
    public class ServiceBusInitialization
    {
        private static ISendOnlyBus bus { get; set; }

        public static ISendOnlyBus GetBusInstance()
        {
            if (bus == null)
                InitializeBus();

            return bus;
        }

        private static void InitializeBus()
        {
            BusConfiguration configuration = new BusConfiguration();

            string SourceQueue = string.Empty;
            SourceQueue = ConfigurationManager.AppSettings["DCKissIMGNSBSagaUpdateEndPoint"];
            configuration.EndpointName(SourceQueue);
            configuration.OverrideLocalAddress(SourceQueue);
            configuration.Conventions()
                .DefiningCommandsAs(type => type.GetCustomAttributes(true).Any(t => t.GetType().Name == "CommandAttribute"))
                .DefiningTimeToBeReceivedAs(GetExpiration)
                ;
            configuration.AssembliesToScan(AllAssemblies.Matching("NServiceBus").And("DCKiss.IMG.NSBSagaUpdate"));
            configuration.DoNotCreateQueues();
            configuration.UseTransport<AzureStorageQueueTransport>();
            configuration.UsePersistence<InMemoryPersistence>();

            if (bus == null)
                bus = Bus.CreateSendOnly(configuration);
        }

        //public void Customize(BusConfiguration configuration)
        //{
        //    configuration.Conventions()
        //        .DefiningCommandsAs(type => type.GetCustomAttributes(true).Any(t => t.GetType().Name == "CommandAttribute"))
        //        .DefiningTimeToBeReceivedAs(GetExpiration)
        //        ;
        //    configuration.AssembliesToScan(AllAssemblies.Matching("NServiceBus").And("DCKiss.IMG.NSBSagaUpdate"));
        //    configuration.UseTransport<AzureStorageQueueTransport>();
        //    //configuration.EnableOutbox();
        //    configuration.UsePersistence<InMemoryPersistence>();
        //}
        private static TimeSpan GetExpiration(Type type)
        {
            dynamic expiresAttribute = type.GetCustomAttributes(true).SingleOrDefault(t => t.GetType().Name == "ExpiresAttribute");
            return expiresAttribute == null ? TimeSpan.MaxValue : expiresAttribute.ExpiresAfter;
        }
    }
}
