﻿using DCKiss.Image.NSBSaga.Models;
using NServiceBus;
using NServiceBus.Logging;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace DCKiss.Image.NSBSaga.Worker_Test
{
    public class InventoryImageImportSaga :
              Saga<InventoryImageSagaData>,
                  IAmStartedByMessages<StartImportInventoryImagesMessage>,
                  IHandleMessages<UploadImageMessage>,
                  IHandleMessages<CancelImageUploadProcessMessage>
    {
        static ILog log = LogManager.GetLogger<InventoryImageImportSaga>();

        protected override void ConfigureHowToFindSaga(SagaPropertyMapper<InventoryImageSagaData> mapper)
        {
            mapper.ConfigureMapping<StartImportInventoryImagesMessage>(m => m.InventoryId).ToSaga(s => s.InventorySagaId);
            mapper.ConfigureMapping<UploadImageMessage>(message => $"{message.InventoryId}_{message.SecondaryKey}")
              .ToSaga(sagaData => sagaData.InventorySagaId);
        }
        public async Task Handle(CancelImageUploadProcessMessage message, IMessageHandlerContext context)
        {
            log.Info($"Received CancelImageUploadProcessMessage message for InventoryId = '{Data.InventorySagaId}'");
            log.Info("Marking as completed");
            MarkAsComplete();
            await Task.CompletedTask;
        }

        public Task Handle(UploadImageMessage message, IMessageHandlerContext context)
        {
            log.Info($"Received CancelImageUploadProcessMessage message for InventoryId = '{Data.InventorySagaId}'");
            // code to handle order completion
            return Task.CompletedTask;
        }

        public async Task Handle(StartImportInventoryImagesMessage message, IMessageHandlerContext context)
        {
            var secondaryKey = Guid.NewGuid().ToString();
            var timeoutData = new InventoryImageSagaData
            {
                InventorySagaId = message.InventoryId, //+ "_" + secondaryKey,
                UploadUserId = message.UploadUserId,
                UploadPassword = message.UploadPassword,
                ImportImageUrls = message.ImportImageUrls
            };

            await RequestTimeout(context, TimeSpan.FromSeconds(5), timeoutData);
        }

    }
}
